﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using C1.WPF.C1Chart;
using System.Collections.ObjectModel;

namespace Wpf_Chart_MultiPointSelection
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(MainWindow_Loaded);            
        }
        
        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            #region Generate Data Source for DataSeries
            DateTime dt = new DateTime(2012, 12, 01);
            ObservableCollection<DateTime> xValueList = new ObservableCollection<DateTime>();
            for (int i = 0; i < 25; i++)
            {
                DateTime tempDate = dt.AddHours(i);
                xValueList.Add(tempDate);
            }

            ObservableCollection<double> valueList = new ObservableCollection<double>();
            valueList.Add(20030);
            valueList.Add(52402);
            valueList.Add(40232);
            valueList.Add(21030);
            valueList.Add(10203);

            valueList.Add(92390);
            valueList.Add(20030);
            valueList.Add(89002);
            valueList.Add(30032);
            valueList.Add(45000);

            valueList.Add(60303);
            valueList.Add(73230);
            valueList.Add(30920);
            valueList.Add(23090);
            valueList.Add(20030);

            valueList.Add(23903);
            valueList.Add(20300);
            valueList.Add(10900);
            valueList.Add(73200);
            valueList.Add(90230);

            valueList.Add(21030);
            valueList.Add(45000);
            valueList.Add(39020);
            valueList.Add(16050);
            valueList.Add(20300);


            #endregion

            # region DataSeries
            XYDataSeries ds = new XYDataSeries();
            ds.XValuesSource = xValueList;
            ds.ValuesSource = valueList;
            ds.ChartType = ChartType.Column;
            c1Chart1.Data.Children.Add(ds);
            ds.PlotElementLoaded += new EventHandler(ds_PlotElementLoaded);
            #endregion

            #region Axis Settings

            c1Chart1.View.AxisX.IsTime = true;
            c1Chart1.View.AxisX.AnnoFormat = "G";
            c1Chart1.View.AxisX.AnnoAngle = 270;

            c1Chart1.View.AxisX.AutoMax = false;
            c1Chart1.View.AxisX.AutoMin = false;
            c1Chart1.View.AxisX.Min = dt.AddHours(0).ToOADate();
            c1Chart1.View.AxisX.Max = dt.AddHours(25).ToOADate();
            c1Chart1.View.AxisX.MajorUnit = TimeSpan.FromHours(1).Hours;
            #endregion
        }

               

        #region ListMode Selection by clicking two Chart Points

        PlotElement startPlotElement; //Reference to first PlotElement in the selected Range
        PlotElement endPlotElement; //Reference to Last PlotElement in the selected Range
        List<PlotElement> selectionList = new List<PlotElement>(); //List to hold the range of bars selected
        SolidColorBrush defaultColor; // Default Color Series used during resetting the Bar properties

        void ds_PlotElementLoaded(object sender, EventArgs e)
        {
            PlotElement ps = sender as PlotElement;
            defaultColor = ps.Fill as SolidColorBrush;

            ps.MouseLeftButtonUp += (s, me) =>
            {
                if (Keyboard.Modifiers == ModifierKeys.Shift)
                {
                    // Assigning reference to first Selected Bar
                    if (startPlotElement == null)
                        startPlotElement = ps;
                    else
                    {
                        // Assigning reference to last Selected Bar
                        endPlotElement = ps;
                        //Create list of bar to be selected 
                        //Based on first and last PlotElement
                        ChartSelectionList();
                    }
                }
                else
                {
                    //Clear existing highlighted bars
                    //When mouse click is done without 
                    ClearSelectionList();
                    startPlotElement = ps;
                    selectionList.Add(startPlotElement);
                }

                ChartPointHighlight();
            };
        }

        /// <summary>
        /// Module defines the Selection of PlotElements
        /// </summary>
        public void ChartSelectionList()
        {

            if (startPlotElement != null && endPlotElement != null)
            {

                int startIndex, endIndex;
                if (startPlotElement.DataPoint.PointIndex < endPlotElement.DataPoint.PointIndex)
                {
                    startIndex = startPlotElement.DataPoint.PointIndex;
                    endIndex = endPlotElement.DataPoint.PointIndex;
                }
                else
                {
                    startIndex = endPlotElement.DataPoint.PointIndex;
                    endIndex = startPlotElement.DataPoint.PointIndex;
                }


                var plotElementList = from ps in (c1Chart1.Data.Children[0] as XYDataSeries).Children.OfType<C1.WPF.C1Chart.Bar>() select ps;

                selectionList = (from ps in plotElementList
                                 where (ps as PlotElement).DataPoint.PointIndex >= startIndex &&
                                     (ps as PlotElement).DataPoint.PointIndex <= endIndex
                                 select ps).ToList<PlotElement>();
            }

        }

        /// <summary>
        /// Highlight Selected Bars
        /// </summary>
        public void ChartPointHighlight()
        {
            foreach (PlotElement ps in selectionList)
            {                
                ps.Fill = new SolidColorBrush(Colors.LightGreen);
                ps.LayoutTransform = new ScaleTransform() { ScaleX = 2 };
            }
        }

        /// <summary>
        /// Remove highlighting and clear Selection of Highlighted Bars
        /// </summary>
        public void ClearSelectionList()
        {
            foreach (PlotElement ps in selectionList)
            {
                ps.Fill = defaultColor;                
                ps.LayoutTransform = new ScaleTransform() { ScaleX = 1 };
                ps.UpdateLayout();
            }

            if (startPlotElement != null)
                if (startPlotElement.Tag != null)
                {
                    startPlotElement.Fill = defaultColor;                    
                    startPlotElement.LayoutTransform = new ScaleTransform() { ScaleX = 1 };
                }

            startPlotElement = null;
            endPlotElement = null;
            selectionList.Clear();

        }

        #endregion

    }
}
